# WEB+DB PRESS Vol.108 特集2「詳解PostgreSQL」サンプルコード

技術評論社刊「WEB+DB PRESS Vol.108」の特集2「詳解PostgreSQL」（曽根壮大著）のサンプルコードです。

# 環境構築
1. PostgreSQLのインストール
2. psqlで接続する
3. CREATE DATABASE demo;
4. psql demo < demo.sql
5. psql -f generate-test-data.sql -d demo

GUIをご利用の場合はsqlファイルの中身を適宜実行していただく形でも問題ありません。
`psql -f generate-test-data.sql -d demo` は1回実行される度にテストデータを1000件程度作成しますので5回程度を目処に複数回実行してください。

[インストールまとめ](https://soudai.hatenablog.com/entry/2018/10/08/023918)

# Window関数
3章の図1~3の同様に下記のSQLを上記の環境で実行する

```SQL
-- データの中身の確認
demo=# SELECT * FROM "売上" WHERE
"商品名" = 'WEB+DB PRESS Vol.105' LIMIT 1;

-- 8〜10月の各月の日別の売上件数の上位3件を表示
demo=# SELECT "売上日", "件数" FROM
 (SELECT "売上日時"::date AS "売上日",
   COUNT(*) AS "件数" FROM "売上"
   WHERE "売上日時" >= '2018/10/01 00:00:00'
     AND "売上日時" <= '2018/10/31 23:59:59'
   GROUP BY "売上日"
ORDER BY "件数" DESC,売上日 LIMIT 3) AS oct
UNION ALL SELECT * FROM
 (SELECT "売上日時"::date AS "売上日",
   COUNT(*) AS "件数" FROM "売上"
   WHERE "売上日時" >= '2018/09/01 00:00:00'
     AND "売上日時" <= '2018/09/30 23:59:59'
   GROUP BY "売上日"
ORDER BY "件数" DESC,売上日 LIMIT 3) AS Sep
UNION ALL SELECT * FROM
 (SELECT "売上日時"::date AS "売上日",
   COUNT(*) AS "件数" FROM "売上"
   WHERE "売上日時" >= '2018/08/01 00:00:00'
     AND "売上日時" <= '2018/08/31 23:59:59'
   GROUP BY "売上日"
ORDER BY "件数" DESC,売上日 LIMIT 3) AS Aug;

--Window関数を利用してランキングを出力
demo=# SELECT "売上日", "件数", "順位"
FROM (
  SELECT *, dense_rank() OVER (PARTITION BY "売上月"
   ORDER BY "件数" DESC, 売上日) AS "順位"
    FROM (
      SELECT
        extract(month from "売上日時") AS "売上月",
        "売上日時"::date AS "売上日",
        COUNT(*) AS "件数"
      FROM "売上"
    WHERE "売上日時" >= '2018/08/01 00:00:00'
      AND "売上日時" <= '2018/10/31 23:59:59'
    GROUP BY "売上日", "売上月"
  ) AS "集計"
) AS "集計ランキング"
WHERE "順位" <= 3
ORDER BY "売上月" DESC, "順位";


-- OVER ~ の振る舞いを確認
demo=# WITH t(id, value) AS (VALUES (1, 1), (2, 1), (3, 3), (4, 5), (5, 5), (6, 5), (7, 6))
SELECT
  id,
  value,
  array_agg(id) OVER ROWS as row_id,
  array_agg(value)  OVER ROWS as row_value,
  array_agg(id) OVER RANGE as renge_id,
  array_agg(value)  OVER RANGE as renge_value,
  array_agg(id) OVER GROUPS as groups_id,
  array_agg(value)  OVER GROUPS  as groups_value
FROM 
  t WINDOW obj AS (ORDER BY value),
  ROWS AS (obj ROWS BETWEEN 1 PRECEDING AND 1 FOLLOWING),
  RANGE AS (obj RANGE BETWEEN 1 PRECEDING AND 1 FOLLOWING),
  GROUPS AS (obj GROUPS BETWEEN 1 PRECEDING AND 1 FOLLOWING);

-- EXCLUDE句を確認
demo=# WITH t(value) AS (VALUES (1), (1), (3), (5), (5), (5), (6))
SELECT
  value,
  array_agg(value) OVER (obj ROWS BETWEEN 1 PRECEDING AND 1 FOLLOWING
      EXCLUDE CURRENT ROW) AS current_row,
  array_agg(value) OVER (obj ROWS BETWEEN 1 PRECEDING AND 1 FOLLOWING
      EXCLUDE GROUP) AS group,
  array_agg(value) OVER (obj ROWS BETWEEN 1 PRECEDING AND 1 FOLLOWING
      EXCLUDE TIES) AS ties,
  array_agg(value) OVER (obj ROWS BETWEEN 1 PRECEDING AND 1 FOLLOWING
      EXCLUDE NO OTHERS) AS no_others
FROM t
WINDOW obj AS (ORDER BY value);
```

# JIT
3章の図4の同様に下記のSQLを上記の環境で実行する

```SQL
-- 通常の実行計画
demo=# EXPLAIN ANALYZE SELECT SUM(relpages) FROM pg_class;

-- JITが利用された場合の実行計画
demo=# SET jit TO on;
demo=# SET jit_above_cost TO 10;

demo=# EXPLAIN ANALYZE SELECT SUM(relpages) FROM pg_class;
```

# パラレルQueryのデモ
3章の図6の同様に下記のSQLを上記の環境で実行する

```SQL
-- パラレル用のデータを作るストアドファンクション
demo=# CREATE FUNCTION public.bigtable() RETURNS SETOF bigint
    LANGUAGE sql ROWS 10000000
    AS $$
SELECT i from generate_series(1, 10000000::bigint) AS i;
$$;

demo=# CREATE TABLE foo (id SERIAL PRIMARY KEY, val1 INTEGER);
-- 10億件のデータ作成
demo=# INSERT INTO foo (id) SELECT bigtable();

demo=# EXPLAIN ANALYZE SELECT id FROM foo WHERE id % 3 = 0;
```

# ストアドファンクション
3章の「ストアドファンクションの役割」の同様に下記のSQLを上記の環境で実行する

```SQL
-- ストアドファンクションでのfizzbuzz
postgres=# CREATE OR REPLACE FUNCTION fizzbuzz()
postgres-# RETURNS setof text as $$
postgres$# BEGIN
postgres$#   FOR i IN 1 .. 100 LOOP
postgres$#     IF MOD(i, 15) = 0 THEN
postgres$#       return next 'FizzBuzz';
postgres$#     ELSIF MOD(i, 5) = 0 THEN
postgres$#       return next 'Buzz';
postgres$#     ELSIF MOD(i, 3) = 0 THEN
postgres$#       return next 'Fizz';
postgres$#     ELSE
postgres$#       return next i::text;
postgres$#     END IF;
postgres$#   END LOOP;
postgres$#   RETURN;
postgres$# END;
postgres$# $$ LANGUAGE plpgsql;
CREATE FUNCTION
postgres=# SELET fizzbuzz();
```

# パーテーション
4章の図5の同様に下記のSQLを上記の環境で実行する

```SQL
-- リストパーテーション
demo=# INSERT INTO "販売履歴" 
  ("売上日時", "商品名", "価格", "売上月")
VALUES (
  '2018-10-21 21:12:00'
  , 'WEB+DB PRESS WEB+DB PRESS Vol.107'
  , 1480
  , '201810'
);
demo=# SELECT * FROM "販売履歴" ;
demo=# SELECT * FROM "2018年10月" ;
demo=# SELECT * FROM "2018年9月" ;

-- レンジパーテーション
demo=# INSERT INTO "販売履歴2" 
  ("売上日時", "商品名", "価格")
VALUES (
  '2018-10-21 21:12:00'
  , 'WEB+DB PRESS WEB+DB PRESS Vol.107'
  , 1480
);

demo=# SELECT * FROM "2018年10月RANGE";

-- 指定していない範囲のINSERTは失敗する
demo=# INSERT INTO "販売履歴2" 
  ("売上日時", "商品名", "価格")
VALUES (
  '2018-11-21 21:12:00'
  , 'WEB+DB PRESS WEB+DB PRESS Vol.107'
  , 1480
);
```